import pkg_resources


def get_version(package_name):
    version = None
    try:
        version = pkg_resources.require(package_name)[0].version
    except pkg_resources.DistributionNotFound:
        module = __import__(package_name)
        try:
            func = getattr(module, 'get_version')
            version = func()
        except:
            for attempt in ('version', 'VERSION', '__version__'):
                try:
                    version = getattr(module, attempt)
                    if isinstance(version, (list, tuple)):
                        version = ".".join(version)
                except AttributeError:
                    pass

    if version:
        return str(version)
    else:
        raise pkg_resources.DistributionNotFound




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'WhatsNew.released'
        db.add_column(u'whatsnew_whatsnew', 'released',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'WhatsNew.released'
        db.delete_column(u'whatsnew_whatsnew', 'released')


    models = {
        u'whatsnew.whatsnew': {
            'Meta': {'object_name': 'WhatsNew'},
            'content': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'released': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        }
    }

    complete_apps = ['whatsnew']


# -*- coding: utf-8 -*-
from django import forms
from django.contrib.admin import ModelAdmin, site
from whatsnew.models import WhatsNew

try:
    from suit_redactor.widgets import RedactorWidget
except ImportError:
    class RedactorWidget(forms.Textarea):
        def __init__(self, attrs=None, editor_options=None):
            super(RedactorWidget, self).__init__(attrs)


class WhatsNewForm(forms.ModelForm):
    class Meta:
        widgets = {
            'content': RedactorWidget(editor_options={'lang': 'en'})
        }


class WhatsNewAdmin(ModelAdmin):
    change_form_template = 'admin/whatsnew/change_form_whatsnew.html'
    list_display = ('version', 'expire', 'enabled')
    form = WhatsNewForm


site.register(WhatsNew, WhatsNewAdmin)



# -*- coding: utf-8 -*-
from django.core import exceptions
from django.db import models
from types import StringTypes
from django.utils.translation import gettext as _
import pkg_resources


class Version(object):
    def __init__(self, vstring):
        self._components = pkg_resources.parse_version(vstring)
        self.string = vstring
        self._patch = []
        for i, part in enumerate(self._components):
            if '*' in part:
                break
        if len(self._components) > i:
            self._patch = map(lambda x: x.replace('*', ''), self._components[i + 1:-1])
        parts = list(self._components[:i])
        parts.extend('0' * (3 - len(parts)))

        self._parts = map(int, parts)

        self._status = self._components[i][1]

    def same(self, other):
        return self._parts == other._parts

    def __str__(self):
        return self.string

    def __repr__(self):
        return "Version ('%s')" % str(self)

    def __unicode__(self):
        return unicode(str(self))

    def __cmp__(self, other):
        if isinstance(other, StringTypes):
            other = Version(other)
        if not other:
            return None
        return cmp(self._components, other._components)

    @property
    def major(self):
        return self._parts[0]

    @property
    def minor(self):
        return self._parts[1]

    @property
    def release(self):
        return self._parts[2]

    @property
    def patch(self):
        return ".".join(self._patch)

    @property
    def status(self):
        if self._status == '@':
            return "nighlybuild"
        return self._status

    @property
    def version(self):
        return self._parts + [self.status] + [self.patch]

class VersionField(models.CharField):
    __metaclass__ = models.SubfieldBase
    default_error_messages = {
        'invalid': _("Enter a valid version number in X.Y.Z format."),
    }
    description = _("Version")

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 50)
        super(VersionField, self).__init__(*args, **kwargs)

    def get_prep_value(self, obj):
        return str(obj)

    def to_python(self, value):
        if value is None or value == '':
            return None
        if isinstance(value, Version):
            return value
        return Version(value)

    def get_db_prep_value(self, value, connection, prepared=False):
        if not prepared:
            value = self.get_prep_value(value)
        return value

    def value_to_string(self, obj):
        value = self.to_python(self._get_val_from_obj(obj))
        return str(value)

    def run_validators(self, value):
        return super(VersionField, self).run_validators(str(value))

    def validate(self, value, model_instance):
        if not self.editable:
            return
        if value is None and not self.null:
            raise exceptions.ValidationError(self.error_messages['null'], code='null')

        if not self.blank and str(value) in self.empty_values:
            raise exceptions.ValidationError(self.error_messages['blank'], code='blank')

        if not (value.major or value.minor or value.release):
            raise exceptions.ValidationError(self.error_messages['invalid'], code='invalid')



def add_south_rules():
    from south.modelsinspector import add_introspection_rules

    add_introspection_rules([
        (
            (VersionField,),
            [],
            {},
        ),
    ], ["whatsnew\.fields"])


try:  # pragma: no cover
    import south
except ImportError: # pragma: no cover
    south = None

if south:  # pragma: no cover
    add_south_rules()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.template import Library
from types import StringTypes
import datetime
from whatsnew.models import WhatsNew
from django.db.models import Q
from whatsnew.fields import Version
from whatsnew.util import get_version


register = Library()


@register.inclusion_tag('whatsnew/whatsnew.html', takes_context=True)
def whatsnew(context, package_name, force=1):
    request = context['request']
    cookie_name = "{}-whatsnew".format(package_name)
    ctx = {'name': cookie_name}
    today = datetime.datetime.today()
    last = None

    last_seen = Version(request.COOKIES.get(cookie_name) or '0.0')
    current_version = Version(get_version(package_name))
    try:
        last = WhatsNew.objects.filter(enabled=True).filter(Q(expire__gte=today) | Q(expire=None)).latest()
        ctx.update({'content': last.content, 'version': news.version, 'display': True})
    except WhatsNew.DoesNotExist:
        ctx.update({'content': None, 'version': None, 'display': force})

    return ctx



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'WhatsNew'
        db.create_table(u'whatsnew_whatsnew', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('version', self.gf('django.db.models.fields.CharField')(max_length=30)),
            ('content', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'whatsnew', ['WhatsNew'])


    def backwards(self, orm):
        # Deleting model 'WhatsNew'
        db.delete_table(u'whatsnew_whatsnew')


    models = {
        u'whatsnew.whatsnew': {
            'Meta': {'object_name': 'WhatsNew'},
            'content': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        }
    }

    complete_apps = ['whatsnew']


# -*- coding: utf-8 -*-
from django.db import models
from .fields import VersionField
from django.utils.translation import gettext as _


class WhatsNew(models.Model):
    version = VersionField()
    content = models.TextField()
    expire = models.DateField(blank=True, null=True)
    enabled = models.BooleanField(default=False)

    def __unicode__(self):
        return unicode(self.version)

    class Meta:
        get_latest_by = 'id'
        verbose_name = _("What's New")
        verbose_name_plural = _("What's New")



#!/usr/bin/env python
import os
import codecs
from distutils.config import PyPIRCCommand
from setuptools import setup, find_packages

dirname = 'whatsnew'

app = __import__(dirname)

setup(
    name=app.NAME,
    version=app.get_version(),
    url='https://github.com/saxix/django-whatsnew',
    description="Simple application to manage `what's new` screen.",
    author='sax',
    author_email='sax@os4d.org',
    license='BSD',
    packages=find_packages('.'),
    include_package_data=True,
    platforms=['linux'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Developers'
    ]
)



from django.conf.urls import patterns, include, url
from django.views.generic import TemplateView


urlpatterns = patterns(
    'whatsnew.views',
    url(r'^test/', TemplateView.as_view(template_name='whatsnew/test.html'), name='whatsnew-test'),
    # url(r'^latest/$', 'latest', name='whatsnew-latest'),
)


